import click

from .requirements import Requirements

@click.group()
def cli():
    cli.rqr = Requirements()

@cli.command()
def list():
    click.echo(cli.rqr)

@cli.command()
@click.argument('pkgs', nargs=-1)
@click.option('--save', 'target', flag_value='base')
@click.option('--save-development', 'target', flag_value='development')
@click.option('--save-production', 'target', flag_value='production')
def install(pkgs, target = None):
    click.echo(cli.rqr.install(pkgs, target))

@cli.command()
def migrate():
    cli.rqr.migrate()

@cli.command()
@click.option('--save/--dry-run', default=True)
def update(save):
    updates = cli.rqr.update(save)
    if len(updates) is 0:
        click.echo('No updates found.')
    for update in updates:
        click.echo('{0} {1} -> {2}'.format(*update))



import requests
from packaging.version import Version

from .serializer import serialize

class Updater:
    def update(self, pkgs):
        updates = []
        for target in pkgs:
            for pkg, ver in pkgs[target].items():
                current_version = Version(ver)
                lastest_version = self.get_last_version(pkg)
                if lastest_version > current_version:
                    updates.append((pkg, current_version, lastest_version))
                    pkgs[target][pkg] = str(lastest_version)
                else:
                    pkgs[target][pkg] = str(current_version)
        return (pkgs, updates)

    def get_last_version(self, pkg_name):
        response = requests.get('https://pypi.python.org/pypi/{0}/json'.format(pkg_name))
        pkginfo = response.json()

        releases = pkginfo['releases']
        for release in sorted(releases.keys(), reverse=True):
            version = Version(release)
            if not version.is_prerelease:
                return version

def check_file_for_updates(filename):
    requirements = {'base': {}}
    f = open(filename, 'r')
    line = True
    while line:
        line = f.readline().strip()
        parts = line.split('==')
        if len(parts) is 2:
            pkg_name, version_name = parts
            current_version = Version(version_name)
            latest_version = get_last_version(pkg_name)

            if latest_version > current_version:
                print(pkg_name, current_version, '->', latest_version)
                requirements['base'][pkg_name] = str(latest_version)
            else:
                print(pkg_name, 'ok')
                requirements['base'][pkg_name] = str(current_version)
        elif len(line.strip()) is not 0:
            print('Invalid line', line)
        # else: empty line
    serialize(requirements)



import re
import os

TARGET_PRODUCTION_PATTERN = '(prod|production)'
TARGET_PRODUCTION_REGEX = re.compile('.*{0}.*'.format(TARGET_PRODUCTION_PATTERN))
TARGET_DEVELOPMENT_PATTERN = '(dev|development)'
TARGET_DEVELOPMENT_REGEX = re.compile('.*{0}.*'.format(TARGET_DEVELOPMENT_PATTERN))
TARGETS_PATTERN = '(base|{0}|{1})'.format(TARGET_PRODUCTION_PATTERN, TARGET_DEVELOPMENT_PATTERN)
REQUIREMENTS_REGEX = re.compile('({0}-)?requirements(-{0})?.txt'.format(TARGETS_PATTERN))
TARGETS_REGEX = re.compile('{0}.txt'.format(TARGETS_PATTERN))
REQUIREMENTS_FOLDER_NAME = 'requirements'

class Migrator:
    def run(self):
        self.pkgs = {
            'base': {},
            'development': {},
            'production': {},
        }
        self.search()
        return self.pkgs

    def search(self):
        for item in os.listdir(os.getcwd()):
            if os.path.isdir(item) and item == REQUIREMENTS_FOLDER_NAME:
                for subitem in os.listdir(item):
                    if TARGETS_REGEX.match(subitem):
                        self.migrate_file(os.path.join(REQUIREMENTS_FOLDER_NAME, subitem))
            elif os.path.isfile(item) and REQUIREMENTS_REGEX.match(item):
                self.migrate_file(item)

    def discover_target(self, filename):
        if TARGET_PRODUCTION_REGEX.match(filename):
            return 'production'
        elif TARGET_DEVELOPMENT_REGEX.match(filename):
            return 'development'
        else:
            return 'base'

    def migrate_file(self, filename):
        target = self.discover_target(filename)
        print('Discovered {0} ({1})'.format(filename, target))
        with open(filename, 'r') as stream:
            line = True
            while line:
                line = stream.readline().strip()
                if line:
                    self.parse_file_line(target, line)
            stream.close()

    def parse_file_line(self, target, line):
        # TODO: do discover other pinned versions not just exact matches
        parts = line.split('==')
        if len(parts) is 2:
            pkg, version = parts
            self.pkgs[target][pkg] = version
            print('  - {0}@{1}'.format(pkg, version))
        else:
            print('  - Invalid:', line)
